/* ******************************************************************************

	FILE:			SSUtils.cpp
	AUTHOR:			Jean-Sbastien Perrier

	DESCRIPTION:	This file contains a collection of utility classes and
					functions usefull for a screen saver application. 
					Yes, I know, the classes should have been in a different 
					file, sorry for not following the coding standards.

	$History::                                                                  $

****************************************************************************** */

// Includes ****************************************************************** //
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <regstr.h>
#include <stdlib.h>
#include <string.h>

#include "SSUtils.h"

// Defines ******************************************************************* //

// Types ********************************************************************* //

// Consts ******************************************************************** //

// Globals ******************************************************************* //

// Static Members Storage Point ********************************************** //

float SSClock::s_fPeriod = 0;	// Period, in seconds, of the performance counter.

// Function Prototypes ******************************************************* //

// Module External Functions ************************************************* //

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	SSUtl_GetArg
// DESCRIPTION:	Isolate the argument (or token) contained in the string.
//				The delimiters are given as a string.
// PARAMETERS:	(in) a_szString	-	Source string.
//				(in) a_szDest	-	Destination string where the token is copied.
//				(in) a_szDelimiters	-	String of delimiters.
// RETURN VALUE: Pointer to the start of the next token in the source string.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
char *SSUtl_GetArg(char *a_szString, char *a_szDest, const char *a_szDelimiters)
{
	while(!strchr(a_szDelimiters, *a_szString) && *a_szString)
	{
		*a_szDest = *a_szString;
		a_szDest++;
		a_szString++;
	}
	// Put the end of string to the token
	*a_szDest = 0;

	if(*a_szString)
	{	// Go to the start of next argument
		while(strchr(a_szDelimiters, *a_szString)) a_szString++;
		// check again if this is the end of string
		if(*a_szString)	return a_szString;
		else return NULL;
	}
	else return NULL;
}
	
/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	SSUtl_VerifyPassword
// DESCRIPTION:	Verify the password.
// PARAMETERS:	(in) hwnd	-	Window handle.
// RETURN VALUE: True or false.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
BOOL SSUtl_VerifyPassword(HWND hwnd)
{
	// Under NT, we return true immediately. This lets the saver quit, and the system manages passwords.
	// Under '95, we call VerifyScreenSavePwd. This checks the appropriate registry key and, if necessary, pops up a verify dialog
	OSVERSIONINFO osv;
	osv.dwOSVersionInfoSize = sizeof(osv);
	
	GetVersionEx(&osv);

	if( osv.dwPlatformId==VER_PLATFORM_WIN32_NT )
		return true;

	HINSTANCE hpwdcpl = ::LoadLibrary("PASSWORD.CPL");
	if( hpwdcpl==NULL )
		return true;

	typedef BOOL (WINAPI *VERIFYSCREENSAVEPWD)(HWND hwnd);
	VERIFYSCREENSAVEPWD VerifyScreenSavePwd;
	VerifyScreenSavePwd = (VERIFYSCREENSAVEPWD)GetProcAddress(hpwdcpl,"VerifyScreenSavePwd");

	if( VerifyScreenSavePwd==NULL )
	{
		FreeLibrary(hpwdcpl);
		return true;
	}
	
	BOOL bres = VerifyScreenSavePwd(hwnd);
	FreeLibrary(hpwdcpl);
	return bres;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	SSUtl_ChangePassword
// DESCRIPTION:	Change the password for this screen saver.
// PARAMETERS:	(in) hwnd	-	Window handle.
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSUtl_ChangePassword(HWND hwnd)
{
	// This only ever gets called under '95, when started with the /a option.
	HINSTANCE hmpr = ::LoadLibrary("MPR.DLL");
	if( hmpr==NULL )
		return;
	
	typedef VOID (WINAPI *PWDCHANGEPASSWORD) (LPCSTR lpcRegkeyname,HWND hwnd,UINT uiReserved1,UINT uiReserved2);
	PWDCHANGEPASSWORD PwdChangePassword=(PWDCHANGEPASSWORD)::GetProcAddress(hmpr,"PwdChangePasswordA");
	if( PwdChangePassword==NULL )
	{
		FreeLibrary(hmpr);
		return;
	}
	
	PwdChangePassword("SCRSAVE",hwnd,0,0); FreeLibrary(hmpr);
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Open
// DESCRIPTION:	Open the registry with the current key path.
// PARAMETERS:	(in) a_szPath	-	key path to open.
// RETURN VALUE: true if it succeded. false otherwise.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
bool SSUtlReg::Open(const char *a_szPath)
{
	LONG res = RegOpenKeyEx(HKEY_CURRENT_USER,a_szPath,0,KEY_ALL_ACCESS,&m_hKey);
	if(res!=ERROR_SUCCESS)	return false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Create
// DESCRIPTION:	Create a registry, opens it if it already exists.
// PARAMETERS:	(in) a_szPath	-	Path of the key to open.
// RETURN VALUE: true on success, false otherwise.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
bool SSUtlReg::Create(const char *a_szPath)
{
	LONG res;	
	DWORD disp;
	res = RegCreateKeyEx(HKEY_CURRENT_USER,a_szPath,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&m_hKey,&disp);
	/* if(disp == REG_CREATED_NEW_KEY)
	{
		// The key did not exist before and was created.
	}
	else
	{
		// The key existed before, it was simply opened.
	}*/
	if( res!=ERROR_SUCCESS ) return false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Close
// DESCRIPTION:	Close the registry key.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSUtlReg::Close()
{
	RegCloseKey(m_hKey);
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	ReadDWord
// DESCRIPTION:	Read a double word (32 bit value) from the registry of the
//				given key name.
// PARAMETERS:	(in) a_szKey	-	Key name to get the value from.
//				(in) a_dwDefault	-	Default value if not found.
// RETURN VALUE: Value read.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
DWORD SSUtlReg::ReadDWord(const char *a_szKey, DWORD a_dwDefault)
{
	DWORD valtype, valsize, val;
	valsize = sizeof(val);
	LONG res = RegQueryValueEx(m_hKey, a_szKey , 0, &valtype, (LPBYTE)&val, &valsize);
	return (res == ERROR_SUCCESS) ? val : a_dwDefault;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	WriteDWord
// DESCRIPTION:	Write a value in the given key.
// PARAMETERS:	(in) a_szKey	-	Key name.
//				(in) a_dwValue	-	Value to write into the key.
// RETURN VALUE: true if the write was sucessfull, false otherwise.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
bool SSUtlReg::WriteDWord(const char *a_szKey, DWORD a_dwValue)
{
	LONG res = RegSetValueEx(m_hKey, a_szKey, 0, REG_DWORD, (CONST BYTE*)&a_dwValue, sizeof(DWORD));
	return (res == ERROR_SUCCESS) ? true : false;
}


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	ReadString
// DESCRIPTION:	Read a null terminated string from registry.
// PARAMETERS:	(in) a_szKey		-	Key name
//				(out) a_szString	-	Buffer that will hold the string
//				(in) a_nDim			-	Size of the buffer.
// RETURN VALUE: true if read was ok. False otherwise. 
// REMARKS:		It may be possible that the buffer is not big enought,
//				in that case, the user has to create a bigger buffer.
/////////////////////////////////////////////////////////////////////////////////
bool SSUtlReg::ReadString(const char *a_szKey, char *a_szString, int a_nDim)
{
	DWORD valtype, valsize;
	valsize = a_nDim;
	LONG res = RegQueryValueEx(m_hKey, a_szKey , 0, &valtype, (LPBYTE)a_szString, &valsize);
	return (res == ERROR_SUCCESS) ? true : false;
}


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	WriteString
// DESCRIPTION:	Write a null-terminated string to registry
// PARAMETERS:	(in) a_szKey	-	Key name
//				(in) a_szString	-	Null terminating String
// RETURN VALUE: true if write successfull, false otherwise.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
bool SSUtlReg::WriteString(const char *a_szKey, const char *a_szString)
{
	DWORD valsize = strlen(a_szString) + 1;
	LONG res = RegSetValueEx(m_hKey, a_szKey, 0, REG_SZ, (CONST BYTE*)a_szString, valsize);
	return (res == ERROR_SUCCESS) ? true : false;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Init
// DESCRIPTION:	Get the performance counter's period in seconds.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSClock::Init()
{
	// Get the performance counter frequency
	LARGE_INTEGER Frequency;
	// The frequency is in Hz (1/sec).
	QueryPerformanceFrequency(&Frequency);	
	// Transform the frequency into a floating point value
	float fFrequency = (float)Frequency.QuadPart;
	// Get the period in seconds.
	s_fPeriod = 1/fFrequency;
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	GetTime
// DESCRIPTION:	Query the performance counter and transform it into seconds.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
float SSClock::GetTime()
{
	LARGE_INTEGER Counter;	
	QueryPerformanceCounter(&Counter);
	float fCounter = (float)Counter.QuadPart;
	return fCounter*s_fPeriod;
}


// Module Internal Functions ************************************************* //

